/*
 *
 *  BlueZ - Bluetooth protocol stack for Linux
 *
 *  Copyright (C) 2005-2007  Marcel Holtmann <marcel@holtmann.org>
 *
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 *
 */

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include <string.h>

#include <dbus/dbus-glib.h>

#include <glib/gi18n.h>

#include <gtk/gtk.h>

#include "client.h"

#include "dbus-glue.h"

static BluetoothClient *client;

static gchar *address = NULL;
static gchar *passkey = "123456";

static GtkWidget *page_search = NULL;
static GtkWidget *page_pairing = NULL;
static GtkWidget *page_summary = NULL;

static GtkTreeModel *model = NULL;
static GtkWidget *label_pairing = NULL;
static GtkWidget *label_passkey = NULL;

static gboolean passkey_agent_request(GObject *agent,
				const char *path, const char *address,
					DBusGMethodInvocation *context)
{
	const char *value = passkey;

	/* Apple Wireless and Mighty Mouse */
	if (strncasecmp(address, "00:0A:95:", 9) == 0 ||
				strncasecmp(address, "00:14:51:", 9) == 0)
		value = "0000";

	dbus_g_method_return(context, value);

	return TRUE;
}

static gboolean passkey_agent_confirm(GObject *agent,
			const char *path, const char *address,
			const char *value, DBusGMethodInvocation *context)
{
	dbus_g_method_return(context);

	return TRUE;
}

static gboolean passkey_agent_cancel(GObject *agent,
			const char *path, const char *address, GError **error)
{
	return TRUE;
}

static gboolean passkey_agent_release(GObject *agent, GError **error)
{
	return TRUE;
}

#include "passkey-agent-glue.h"

#if 0
static gint page_forward(gint current_page, gpointer data)
{
	return current_page + 1;
}
#endif

static void close_callback(GtkWidget *assistant, gpointer data)
{
	gtk_widget_destroy(assistant);

	gtk_main_quit();
}

static void cancel_callback(GtkWidget *assistant, gpointer data)
{
	gtk_widget_destroy(assistant);

	gtk_main_quit();
}

static void prepare_callback(GtkWidget *assistant, GtkWidget *page, gpointer data)
{
	if (page == page_search) {
		bluetooth_client_discover_devices(client, "/org/bluez/hci0");
		return;
	}

	if (page == page_pairing) {
		bluetooth_client_cancel_discovery(client, "/org/bluez/hci0");

		bluetooth_client_register_passkey_agent(client,
					"/org/bluez/applet", address,
					&dbus_glib_passkey_agent_object_info);

		bluetooth_client_create_bonding(client,
					"/org/bluez/hci0", address);

		gtk_label_set_markup(GTK_LABEL(label_pairing), address);
		gtk_label_set_markup(GTK_LABEL(label_passkey), passkey);
		return;
	}

	gtk_assistant_set_page_complete(GTK_ASSISTANT(assistant), page, TRUE);
}

static GtkWidget *create_vbox(GtkWidget *assistant, GtkAssistantPageType type,
				const gchar *title, const gchar *section)
{
	GtkWidget *vbox;
	GtkWidget *label;
	GdkPixbuf *pixbuf;
	gchar *str;

	vbox = gtk_vbox_new(FALSE, 6);

	gtk_container_set_border_width(GTK_CONTAINER(vbox), 24);

	gtk_assistant_append_page(GTK_ASSISTANT(assistant), vbox);

	gtk_assistant_set_page_type(GTK_ASSISTANT(assistant), vbox, type);

	str = g_strdup_printf(" %s", title);

	gtk_assistant_set_page_title(GTK_ASSISTANT(assistant), vbox, str);

	g_free(str);

	//pixbuf = gtk_widget_render_icon(GTK_WIDGET(assistant),
	//		GTK_STOCK_DIALOG_INFO, GTK_ICON_SIZE_DIALOG, NULL);

	pixbuf = gtk_icon_theme_load_icon(gtk_icon_theme_get_default(),
					"stock_bluetooth", 32, 0, NULL);

	gtk_assistant_set_page_header_image(GTK_ASSISTANT(assistant),
								vbox, pixbuf);

	g_object_unref(pixbuf);

	if (section) {
		label = gtk_label_new(NULL);

		str = g_strdup_printf("<b>%s</b>\n", section);

		gtk_label_set_markup(GTK_LABEL(label), str);

		g_free(str);

		gtk_misc_set_alignment(GTK_MISC(label), 0, 0);

		gtk_box_pack_start(GTK_BOX(vbox), label, FALSE, FALSE, 0);
	}

	return vbox;
}

static void create_intro(GtkWidget *assistant)
{
	GtkWidget *vbox;
	GtkWidget *label;
	GtkWidget *combo;
	GtkTreeModel *model;
	GtkCellRenderer *renderer;

	vbox = create_vbox(assistant, GTK_ASSISTANT_PAGE_INTRO,
			_("Introduction"),
			_("Welcome to the Bluetooth device setup wizard"));

	label = gtk_label_new(NULL);

	gtk_label_set_markup(GTK_LABEL(label), _("The device wizard will "
				"walk you through the process of configuring "
				"Bluetooth enabled devices for use with this "
				"computer.\n\n"));

	gtk_misc_set_alignment(GTK_MISC(label), 0, 0);

	gtk_label_set_line_wrap(GTK_LABEL(label), TRUE);

	gtk_widget_set_size_request(GTK_WIDGET(label), 380, -1);

	gtk_box_pack_start(GTK_BOX(vbox), label, FALSE, TRUE, 0);

	combo = gtk_combo_box_new();

	model = bluetooth_client_get_model_adapter_list(client);

	gtk_combo_box_set_model(GTK_COMBO_BOX(combo), model);

	g_object_unref(model);

	gtk_combo_box_set_active(GTK_COMBO_BOX(combo), 0);

	gtk_cell_layout_clear(GTK_CELL_LAYOUT(combo));

	renderer = gtk_cell_renderer_text_new();

	gtk_cell_layout_pack_start(GTK_CELL_LAYOUT(combo), renderer, TRUE);

	gtk_cell_layout_set_attributes(GTK_CELL_LAYOUT(combo), renderer,
						"text", COLUMN_NAME, NULL);

	if (gtk_tree_model_iter_n_children(model, NULL) > 1)
		gtk_box_pack_start(GTK_BOX(vbox), combo, FALSE, FALSE, 0);
}

static void create_type(GtkWidget *assistant)
{
	GtkWidget *vbox;
	GtkWidget *button;
	GSList *group = NULL;

	vbox = create_vbox(assistant, GTK_ASSISTANT_PAGE_CONTENT,
				_("Device type"),
				_("Select type of device you wanna setup"));

	button = gtk_radio_button_new_with_label(group, _("Mouse"));

	group = gtk_radio_button_get_group(GTK_RADIO_BUTTON(button));

	gtk_widget_set_sensitive(GTK_WIDGET(button), FALSE);

	gtk_box_pack_start(GTK_BOX(vbox), button, FALSE, FALSE, 0);

	button = gtk_radio_button_new_with_label(group, _("Keyboard"));

	group = gtk_radio_button_get_group(GTK_RADIO_BUTTON(button));

	gtk_widget_set_sensitive(GTK_WIDGET(button), FALSE);

	gtk_box_pack_start(GTK_BOX(vbox), button, FALSE, FALSE, 0);

	button = gtk_radio_button_new_with_label(group, _("Mobile phone"));

	group = gtk_radio_button_get_group(GTK_RADIO_BUTTON(button));

	gtk_widget_set_sensitive(GTK_WIDGET(button), FALSE);

	gtk_box_pack_start(GTK_BOX(vbox), button, FALSE, FALSE, 0);

	button = gtk_radio_button_new_with_label(group, _("Printer"));

	group = gtk_radio_button_get_group(GTK_RADIO_BUTTON(button));

	gtk_widget_set_sensitive(GTK_WIDGET(button), FALSE);

	gtk_box_pack_start(GTK_BOX(vbox), button, FALSE, FALSE, 0);

	button = gtk_radio_button_new_with_label(group, _("Headset"));

	group = gtk_radio_button_get_group(GTK_RADIO_BUTTON(button));

	gtk_widget_set_sensitive(GTK_WIDGET(button), FALSE);

	gtk_box_pack_start(GTK_BOX(vbox), button, FALSE, FALSE, 0);

	button = gtk_radio_button_new_with_label(group, _("Any device"));

	group = gtk_radio_button_get_group(GTK_RADIO_BUTTON(button));

	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(button), TRUE);

	gtk_box_pack_start(GTK_BOX(vbox), button, FALSE, FALSE, 0);
}

static void select_callback(GtkTreeSelection *selection, gpointer user_data)
{
	GtkAssistant *assistant = user_data;
	GtkTreeIter iter;
	gboolean sensitive;

	sensitive = gtk_tree_selection_get_selected(selection, NULL, &iter);

	if (sensitive == TRUE) {
		g_free(address);

		address = NULL;

		gtk_tree_model_get(model, &iter, COLUMN_ADDRESS, &address, -1);
	}

	gtk_assistant_set_page_complete(assistant, page_search, sensitive);
}

static void name_to_text(GtkTreeViewColumn *column, GtkCellRenderer *cell,
			GtkTreeModel *model, GtkTreeIter *iter, gpointer data)
{
	gchar *address;
	gchar *name;

	gtk_tree_model_get(model, iter, COLUMN_ADDRESS, &address,
						COLUMN_NAME, &name, -1);

	g_object_set(cell, "text", name ? name : address, NULL);

	g_free(name);
	g_free(address);
}

static void type_to_icon(GtkTreeViewColumn *column, GtkCellRenderer *cell,
			GtkTreeModel *model, GtkTreeIter *iter, gpointer data)
{
	guint type;

	gtk_tree_model_get(model, iter, COLUMN_TYPE, &type, -1);

	switch (type) {
	case BLUETOOTH_TYPE_PHONE:
		g_object_set(cell, "icon-name", "stock_cell-phone", NULL);
		break;
	case BLUETOOTH_TYPE_MODEM:
		g_object_set(cell, "icon-name", "modem", NULL);
		break;
	case BLUETOOTH_TYPE_COMPUTER:
		g_object_set(cell, "icon-name", "computer", NULL);
		break;
	case BLUETOOTH_TYPE_NETWORK:
		g_object_set(cell, "icon-name", "network-wireless", NULL);
		break;
	case BLUETOOTH_TYPE_HEADSET:
		g_object_set(cell, "icon-name", "stock_headphones", NULL);
		break;
	case BLUETOOTH_TYPE_KEYBOARD:
		g_object_set(cell, "icon-name", "input-keyboard", NULL);
		break;
	case BLUETOOTH_TYPE_MOUSE:
		g_object_set(cell, "icon-name", "input-mouse", NULL);
		break;
	case BLUETOOTH_TYPE_CAMERA:
		g_object_set(cell, "icon-name", "camera-photo", NULL);
		break;
	case BLUETOOTH_TYPE_PRINTER:
		g_object_set(cell, "icon-name", "printer", NULL);
		break;
	default:
		g_object_set(cell, "icon-name", "stock_bluetooth", NULL);
		break;
	}
}

static void type_to_text(GtkTreeViewColumn *column, GtkCellRenderer *cell,
			GtkTreeModel *model, GtkTreeIter *iter, gpointer data)
{
	guint type;

	gtk_tree_model_get(model, iter, COLUMN_TYPE, &type, -1);

	g_object_set(cell, "text", bluetooth_type_to_string(type), NULL);
}

static void create_search(GtkWidget *assistant)
{
	GtkWidget *vbox;
	GtkWidget *tree;
	GtkWidget *scrolled;
	GtkCellRenderer *renderer;
	GtkTreeViewColumn *column;
	GtkTreeSelection *selection;

	vbox = create_vbox(assistant, GTK_ASSISTANT_PAGE_CONTENT,
				_("Device search"),
				_("Select device you wanna setup"));

	scrolled = gtk_scrolled_window_new(NULL, NULL);

	gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(scrolled),
				GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);

	gtk_scrolled_window_set_shadow_type(GTK_SCROLLED_WINDOW(scrolled),
							GTK_SHADOW_OUT);

	gtk_container_add(GTK_CONTAINER(vbox), scrolled);

	tree = gtk_tree_view_new();

	gtk_tree_view_set_headers_visible(GTK_TREE_VIEW(tree), TRUE);

	gtk_tree_view_set_rules_hint(GTK_TREE_VIEW(tree), TRUE);

	g_object_set(tree, "show-expanders", FALSE, NULL);

	column = gtk_tree_view_column_new();

	gtk_tree_view_column_set_title(column, "Device");

	renderer = gtk_cell_renderer_pixbuf_new();
	gtk_tree_view_column_set_spacing(column, 4);
	gtk_tree_view_column_pack_start(column, renderer, FALSE);

	gtk_tree_view_column_set_cell_data_func(column, renderer,
						type_to_icon, NULL, NULL);

	renderer = gtk_cell_renderer_text_new();
	gtk_tree_view_column_pack_start(column, renderer, TRUE);

	gtk_tree_view_column_set_cell_data_func(column, renderer,
						name_to_text, NULL, NULL);

	gtk_tree_view_append_column(GTK_TREE_VIEW(tree), column);

	gtk_tree_view_column_set_min_width(GTK_TREE_VIEW_COLUMN(column), 280);

	gtk_tree_view_insert_column_with_data_func(GTK_TREE_VIEW(tree), -1,
					"Type", gtk_cell_renderer_text_new(),
						type_to_text, NULL, NULL);

	model = bluetooth_client_get_model_for_adapter(client, "/org/bluez/hci0");

	gtk_tree_view_set_model(GTK_TREE_VIEW(tree), model);

	g_object_unref(model);

	selection = gtk_tree_view_get_selection(GTK_TREE_VIEW(tree));

	gtk_tree_selection_set_mode(selection, GTK_SELECTION_SINGLE);

	g_signal_connect(G_OBJECT(selection), "changed",
					G_CALLBACK(select_callback), assistant);

	gtk_container_add(GTK_CONTAINER(scrolled), tree);

	page_search = vbox;
}

static void create_pairing(GtkWidget *assistant)
{
	GtkWidget *vbox;
	GtkWidget *label;

	vbox = create_vbox(assistant, GTK_ASSISTANT_PAGE_CONTENT,
				_("Device pairing"),
				_("Pairing with new device"));

	label = gtk_label_new(NULL);

	gtk_misc_set_alignment(GTK_MISC(label), 0, 0);

	gtk_box_pack_start(GTK_BOX(vbox), label, FALSE, TRUE, 0);

	label_pairing = label;

	label = gtk_label_new(NULL);

	gtk_misc_set_alignment(GTK_MISC(label), 0, 0);

	gtk_box_pack_start(GTK_BOX(vbox), label, FALSE, TRUE, 0);

	label_passkey = label;

	page_pairing = vbox;
}

static void create_summary(GtkWidget *assistant)
{
	GtkWidget *vbox;

	vbox = create_vbox(assistant, GTK_ASSISTANT_PAGE_SUMMARY,
				_("Summary"),
				_("Succesfully configured new device"));

	page_summary = vbox;
}

static void create_wizard(void)
{
	GtkWidget *assistant;

	assistant = gtk_assistant_new();

	gtk_window_set_title(GTK_WINDOW(assistant), _("Bluetooth device wizard"));

	gtk_window_set_icon_name(GTK_WINDOW(assistant), "stock_bluetooth");

	gtk_window_set_position(GTK_WINDOW(assistant), GTK_WIN_POS_CENTER);

	gtk_window_set_default_size(GTK_WINDOW(assistant), 440, 440);

#if 0
	gtk_assistant_set_forward_page_func(GTK_ASSISTANT(assistant),
						page_forward, NULL, NULL);
#endif

	create_intro(assistant);

	create_type(assistant);

	create_search(assistant);

	create_pairing(assistant);

	create_summary(assistant);

	g_signal_connect(G_OBJECT(assistant), "close",
					G_CALLBACK(close_callback), NULL);

	g_signal_connect(G_OBJECT(assistant), "cancel",
					G_CALLBACK(cancel_callback), NULL);

	g_signal_connect(G_OBJECT(assistant), "prepare",
					G_CALLBACK(prepare_callback), NULL);

	gtk_widget_show_all(assistant);

	gtk_assistant_update_buttons_state(GTK_ASSISTANT(assistant));
}

int main(int argc, char *argv[])
{
	bindtextdomain(GETTEXT_PACKAGE, LOCALEDIR);
	bind_textdomain_codeset(GETTEXT_PACKAGE, "UTF-8");
	textdomain(GETTEXT_PACKAGE);

	gtk_init(&argc, &argv);

	client = bluetooth_client_new();

	create_wizard();

	gtk_main();

	g_object_unref(client);

	return 0;
}
